import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { ResourceSet } from 'vscode/vscode/vs/base/common/map';
import { marked } from 'marked';
import { Schemas } from 'vscode/vscode/vs/base/common/network';
import { basename } from 'vscode/vscode/vs/base/common/resources';
import { URI } from 'vscode/vscode/vs/base/common/uri';
import { IBulkEditService } from 'vscode/vscode/vs/editor/browser/services/bulkEditService';
import { isLocation } from 'vscode/vscode/vs/editor/common/languages';
import { localize2, localize } from 'vscode/vscode/vs/nls';
import { registerAction2, Action2, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { IConfigurationService } from 'vscode/vscode/vs/platform/configuration/common/configuration.service';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IDialogService } from 'vscode/vscode/vs/platform/dialogs/common/dialogs.service';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService.service';
import { IViewsService } from 'vscode/vscode/vs/workbench/services/views/common/viewsService.service';
import { ResourceNotebookCellEdit } from 'vscode/vscode/vs/workbench/contrib/bulkEdit/browser/bulkCellEdits';
import { MENU_INLINE_CHAT_WIDGET_SECONDARY } from '../../../inlineChat/common/inlineChat.js';
import { NOTEBOOK_EDITOR_ID, CellKind } from 'vscode/vscode/vs/workbench/contrib/notebook/common/notebookCommon';
import { NOTEBOOK_IS_ACTIVE_EDITOR } from 'vscode/vscode/vs/workbench/contrib/notebook/common/notebookContextKeys';
import { ChatAgentLocation } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents';
import { IChatAgentService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents.service';
import { CONTEXT_RESPONSE_VOTE, CONTEXT_RESPONSE, CONTEXT_RESPONSE_ERROR, CONTEXT_CHAT_RESPONSE_SUPPORT_ISSUE_REPORTING, CONTEXT_ITEM_ID, CONTEXT_LAST_ITEM_ID, CONTEXT_RESPONSE_FILTERED, CONTEXT_CHAT_LOCATION, CONTEXT_IN_CHAT_SESSION, CONTEXT_IN_CHAT_INPUT, CONTEXT_REQUEST, CONTEXT_CHAT_EDITING_PARTICIPANT_REGISTERED } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatContextKeys';
import { applyingChatEditsFailedContextKey } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatEditingService';
import { IChatEditingService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatEditingService.service';
import { ChatAgentVoteDirection } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatService';
import { IChatService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatService.service';
import { isResponseVM, isRequestVM } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatViewModel';
import { EDITS_VIEW_ID } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/browser/chat';
import { IChatWidgetService } from 'vscode/vscode/vs/workbench/contrib/chat/browser/chat.service';
import '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/browser/actions/chatActions';
import { CHAT_CATEGORY } from 'vscode/vscode/vs/workbench/contrib/chat/browser/actions/chatConstants';

const MarkUnhelpfulActionId = 'workbench.action.chat.markUnhelpful';
function registerChatTitleActions() {
    registerAction2(class MarkHelpfulAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.markHelpful',
                title: ( localize2(7569, "Helpful")),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.thumbsup,
                toggled: ( (CONTEXT_RESPONSE_VOTE.isEqualTo('up'))),
                menu: [{
                        id: MenuId.ChatMessageFooter,
                        group: 'navigation',
                        order: 1,
                        when: ( (ContextKeyExpr.and(CONTEXT_RESPONSE,  (CONTEXT_RESPONSE_ERROR.negate()))))
                    }, {
                        id: MENU_INLINE_CHAT_WIDGET_SECONDARY,
                        group: 'navigation',
                        order: 1,
                        when: ( (ContextKeyExpr.and(CONTEXT_RESPONSE,  (CONTEXT_RESPONSE_ERROR.negate()))))
                    }]
            });
        }
        run(accessor, ...args) {
            const item = args[0];
            if (!isResponseVM(item)) {
                return;
            }
            const chatService = accessor.get(IChatService);
            chatService.notifyUserAction({
                agentId: item.agent?.id,
                command: item.slashCommand?.name,
                sessionId: item.sessionId,
                requestId: item.requestId,
                result: item.result,
                action: {
                    kind: 'vote',
                    direction: ChatAgentVoteDirection.Up,
                    reason: undefined
                }
            });
            item.setVote(ChatAgentVoteDirection.Up);
            item.setVoteDownReason(undefined);
        }
    });
    registerAction2(class MarkUnhelpfulAction extends Action2 {
        constructor() {
            super({
                id: MarkUnhelpfulActionId,
                title: ( localize2(7570, "Unhelpful")),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.thumbsdown,
                toggled: ( (CONTEXT_RESPONSE_VOTE.isEqualTo('down'))),
                menu: [{
                        id: MenuId.ChatMessageFooter,
                        group: 'navigation',
                        order: 2,
                        when: ( (ContextKeyExpr.and(CONTEXT_RESPONSE)))
                    }, {
                        id: MENU_INLINE_CHAT_WIDGET_SECONDARY,
                        group: 'navigation',
                        order: 2,
                        when: ( (ContextKeyExpr.and(CONTEXT_RESPONSE,  (CONTEXT_RESPONSE_ERROR.negate()))))
                    }]
            });
        }
        run(accessor, ...args) {
            const item = args[0];
            if (!isResponseVM(item)) {
                return;
            }
            const reason = args[1];
            if (typeof reason !== 'string') {
                return;
            }
            item.setVote(ChatAgentVoteDirection.Down);
            item.setVoteDownReason(reason);
            const chatService = accessor.get(IChatService);
            chatService.notifyUserAction({
                agentId: item.agent?.id,
                command: item.slashCommand?.name,
                sessionId: item.sessionId,
                requestId: item.requestId,
                result: item.result,
                action: {
                    kind: 'vote',
                    direction: ChatAgentVoteDirection.Down,
                    reason: item.voteDownReason
                }
            });
        }
    });
    registerAction2(class ReportIssueForBugAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.reportIssueForBug',
                title: ( localize2(7571, "Report Issue")),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.report,
                menu: [{
                        id: MenuId.ChatMessageFooter,
                        group: 'navigation',
                        order: 3,
                        when: ( (ContextKeyExpr.and(CONTEXT_CHAT_RESPONSE_SUPPORT_ISSUE_REPORTING, CONTEXT_RESPONSE)))
                    }, {
                        id: MENU_INLINE_CHAT_WIDGET_SECONDARY,
                        group: 'navigation',
                        order: 3,
                        when: ( (ContextKeyExpr.and(CONTEXT_CHAT_RESPONSE_SUPPORT_ISSUE_REPORTING, CONTEXT_RESPONSE)))
                    }]
            });
        }
        run(accessor, ...args) {
            const item = args[0];
            if (!isResponseVM(item)) {
                return;
            }
            const chatService = accessor.get(IChatService);
            chatService.notifyUserAction({
                agentId: item.agent?.id,
                command: item.slashCommand?.name,
                sessionId: item.sessionId,
                requestId: item.requestId,
                result: item.result,
                action: {
                    kind: 'bug'
                }
            });
        }
    });
    registerAction2(class RetryChatAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.retry',
                title: ( localize2(7572, "Retry")),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.refresh,
                menu: [
                    {
                        id: MenuId.ChatMessageFooter,
                        group: 'navigation',
                        when: ( (ContextKeyExpr.and(
                            CONTEXT_RESPONSE,
                            ContextKeyExpr.in(CONTEXT_ITEM_ID.key, CONTEXT_LAST_ITEM_ID.key)
                        )))
                    },
                    {
                        id: MenuId.ChatEditingWidgetToolbar,
                        group: 'navigation',
                        when: applyingChatEditsFailedContextKey,
                        order: 0
                    }
                ]
            });
        }
        async run(accessor, ...args) {
            const chatWidgetService = accessor.get(IChatWidgetService);
            let item = args[0];
            if (typeof item === 'object' && !!item && 'sessionId' in item) {
                item = chatWidgetService.getWidgetBySessionId(item.sessionId)?.viewModel?.getItems().at(-1);
            }
            if (!isResponseVM(item)) {
                return;
            }
            const chatService = accessor.get(IChatService);
            const chatEditingService = accessor.get(IChatEditingService);
            const chatModel = chatService.getSession(item.sessionId);
            const chatRequests = chatModel?.getRequests();
            if (!chatRequests) {
                return;
            }
            const itemIndex = chatRequests?.findIndex(request => request.id === item.requestId);
            if (chatModel?.initialLocation === ChatAgentLocation.EditingSession) {
                const configurationService = accessor.get(IConfigurationService);
                const dialogService = accessor.get(IDialogService);
                const entriesModifiedInLastRequest = chatEditingService.currentEditingSessionObs.get()?.entries.get().filter((entry) => entry.lastModifyingRequestId === item.requestId) ?? [];
                const shouldPrompt = entriesModifiedInLastRequest.length > 0 && configurationService.getValue('chat.editing.confirmEditRequestRetry') === true;
                const confirmation = shouldPrompt
                    ? await dialogService.confirm({
                        title: ( localize(7573, "Do you want to retry your last request?")),
                        message: entriesModifiedInLastRequest.length === 1
                            ? ( localize(
                            7574,
                            "This will undo edits made to {0} since this request.",
                            basename(entriesModifiedInLastRequest[0].modifiedURI)
                        ))
                            : ( localize(
                            7575,
                            "This will undo edits made to {0} files in your working set since this request. Do you want to proceed?",
                            entriesModifiedInLastRequest.length
                        )),
                        primaryButton: ( localize(7576, "Yes")),
                        checkbox: { label: ( localize(7577, "Don't ask again")), checked: false },
                        type: 'info'
                    })
                    : { confirmed: true };
                if (!confirmation.confirmed) {
                    return;
                }
                if (confirmation.checkboxChecked) {
                    await configurationService.updateValue('chat.editing.confirmEditRequestRetry', false);
                }
                const snapshotRequest = chatRequests[itemIndex];
                if (snapshotRequest) {
                    await chatEditingService.restoreSnapshot(snapshotRequest.id);
                }
            }
            const request = chatModel?.getRequests().find(candidate => candidate.id === item.requestId);
            const languageModelId = chatWidgetService.getWidgetBySessionId(item.sessionId)?.input.currentLanguageModel;
            chatService.resendRequest(request, { userSelectedModelId: languageModelId });
        }
    });
    registerAction2(class InsertToNotebookAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.insertIntoNotebook',
                title: ( localize2(7578, "Insert into Notebook")),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.insert,
                menu: {
                    id: MenuId.ChatMessageFooter,
                    group: 'navigation',
                    isHiddenByDefault: true,
                    when: ( (ContextKeyExpr.and(
                        NOTEBOOK_IS_ACTIVE_EDITOR,
                        CONTEXT_RESPONSE,
                         (CONTEXT_RESPONSE_FILTERED.negate())
                    )))
                }
            });
        }
        async run(accessor, ...args) {
            const item = args[0];
            if (!isResponseVM(item)) {
                return;
            }
            const editorService = accessor.get(IEditorService);
            if (editorService.activeEditorPane?.getId() === NOTEBOOK_EDITOR_ID) {
                const notebookEditor = editorService.activeEditorPane.getControl();
                if (!notebookEditor.hasModel()) {
                    return;
                }
                if (notebookEditor.isReadOnly) {
                    return;
                }
                const value = ( (item.response.toString()));
                const splitContents = splitMarkdownAndCodeBlocks(value);
                const focusRange = notebookEditor.getFocus();
                const index = Math.max(focusRange.end, 0);
                const bulkEditService = accessor.get(IBulkEditService);
                await bulkEditService.apply([
                    ( (new ResourceNotebookCellEdit(notebookEditor.textModel.uri, {
                        editType: 1 ,
                        index: index,
                        count: 0,
                        cells: ( (splitContents.map(content => {
                            const kind = content.type === 'markdown' ? CellKind.Markup : CellKind.Code;
                            const language = content.type === 'markdown' ? 'markdown' : content.language;
                            const mime = content.type === 'markdown' ? 'text/markdown' : `text/x-${content.language}`;
                            return {
                                cellKind: kind,
                                language,
                                mime,
                                source: content.content,
                                outputs: [],
                                metadata: {}
                            };
                        })))
                    })))
                ], { quotableLabel: 'Insert into Notebook' });
            }
        }
    });
    registerAction2(class RemoveAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.remove',
                title: ( localize2(7579, "Remove Request")),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.x,
                precondition: ( (CONTEXT_CHAT_LOCATION.notEqualsTo(ChatAgentLocation.EditingSession))),
                keybinding: {
                    primary: 20 ,
                    mac: {
                        primary: 2048  | 1 ,
                    },
                    when: ( (ContextKeyExpr.and(
                         (CONTEXT_CHAT_LOCATION.notEqualsTo(ChatAgentLocation.EditingSession)),
                        CONTEXT_IN_CHAT_SESSION,
                         (CONTEXT_IN_CHAT_INPUT.negate())
                    ))),
                    weight: 200 ,
                },
                menu: {
                    id: MenuId.ChatMessageTitle,
                    group: 'navigation',
                    order: 2,
                    when: ( (ContextKeyExpr.and(
                         (CONTEXT_CHAT_LOCATION.notEqualsTo(ChatAgentLocation.EditingSession)),
                        CONTEXT_REQUEST
                    )))
                }
            });
        }
        run(accessor, ...args) {
            let item = args[0];
            if (!isRequestVM(item)) {
                const chatWidgetService = accessor.get(IChatWidgetService);
                const widget = chatWidgetService.lastFocusedWidget;
                item = widget?.getFocus();
            }
            if (!item) {
                return;
            }
            const chatService = accessor.get(IChatService);
            const chatModel = chatService.getSession(item.sessionId);
            if (chatModel?.initialLocation === ChatAgentLocation.EditingSession) {
                return;
            }
            const requestId = isRequestVM(item) ? item.id :
                isResponseVM(item) ? item.requestId : undefined;
            if (requestId) {
                const chatService = accessor.get(IChatService);
                chatService.removeRequest(item.sessionId, requestId);
            }
        }
    });
    registerAction2(class ContinueEditingAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.startEditing',
                title: ( localize2(7580, "Edit with Copilot")),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.goToEditingSession,
                precondition: ( (ContextKeyExpr.and(
                    CONTEXT_CHAT_EDITING_PARTICIPANT_REGISTERED,
                     (CONTEXT_CHAT_LOCATION.notEqualsTo(ChatAgentLocation.EditingSession))
                ))),
                menu: {
                    id: MenuId.ChatMessageFooter,
                    group: 'navigation',
                    order: 4,
                    when: ( (ContextKeyExpr.false()))
                }
            });
        }
        async run(accessor, ...args) {
            if (!accessor.get(IChatAgentService).getDefaultAgent(ChatAgentLocation.EditingSession)) {
                return;
            }
            const chatWidgetService = accessor.get(IChatWidgetService);
            const chatService = accessor.get(IChatService);
            const viewsService = accessor.get(IViewsService);
            const dialogService = accessor.get(IDialogService);
            const chatEditingService = accessor.get(IChatEditingService);
            let item = args[0];
            if (!isResponseVM(item)) {
                const widget = chatWidgetService.lastFocusedWidget;
                item = widget?.getFocus();
            }
            if (!item) {
                return;
            }
            const chatModel = chatService.getSession(item.sessionId);
            if (chatModel?.initialLocation === ChatAgentLocation.EditingSession) {
                return;
            }
            const requestId = isRequestVM(item) ? item.id :
                isResponseVM(item) ? item.requestId : undefined;
            const request = chatModel?.getRequests().find(candidate => candidate.id === requestId);
            if (request) {
                const currentEditingSession = chatEditingService.currentEditingSessionObs.get();
                const currentEdits = currentEditingSession?.entries.get();
                const currentEditCount = currentEdits?.length;
                if (currentEditingSession && currentEditCount) {
                    const undecidedEdits = currentEdits.filter((edit) => edit.state.get() === 0 );
                    if (undecidedEdits.length) {
                        const { result } = await dialogService.prompt({
                            title: ( localize(7581, "Start new editing session?")),
                            message: ( localize(
                                7582,
                                "Starting a new editing session will end your current session. Do you want to discard pending edits to {0} files?",
                                undecidedEdits.length
                            )),
                            type: 'info',
                            buttons: [
                                {
                                    label: ( localize(7583, "Discard & Continue")),
                                    run: async () => {
                                        await currentEditingSession.reject();
                                        return true;
                                    }
                                },
                                {
                                    label: ( localize(7584, "Accept & Continue")),
                                    run: async () => {
                                        await currentEditingSession.accept();
                                        return true;
                                    }
                                }
                            ],
                        });
                        if (!result) {
                            return;
                        }
                    }
                    else {
                        const result = await dialogService.confirm({
                            title: ( localize(7581, "Start new editing session?")),
                            message: currentEditCount
                                ? ( localize(
                                7585,
                                "Starting a new editing session will end your current editing session containing {0} file. Do you wish to proceed?",
                                currentEditCount
                            ))
                                : ( localize(
                                7586,
                                "Starting a new editing session will end your current editing session containing {0} files. Do you wish to proceed?",
                                currentEditCount
                            )),
                            type: 'info',
                            primaryButton: ( localize(7587, "Yes"))
                        });
                        if (!result.confirmed) {
                            return;
                        }
                    }
                    await currentEditingSession?.stop();
                    const existingEditingChatWidget = chatWidgetService.getWidgetBySessionId(currentEditingSession.chatSessionId);
                    existingEditingChatWidget?.clear();
                    existingEditingChatWidget?.attachmentModel.clear();
                }
                const { widget } = await viewsService.openView(EDITS_VIEW_ID);
                if (widget.viewModel) {
                    const workingSetInputs = ( (new ResourceSet()));
                    const message = [];
                    for (const item of request.response?.response.value ?? []) {
                        if (item.kind === 'inlineReference') {
                            workingSetInputs.add(isLocation(item.inlineReference) ? item.inlineReference.uri : URI.isUri(item.inlineReference) ? item.inlineReference : item.inlineReference.location.uri);
                        }
                        if (item.kind === 'textEditGroup') {
                            for (const group of item.edits) {
                                message.push({
                                    kind: 'textEdit',
                                    edits: group,
                                    uri: item.uri
                                });
                            }
                        }
                        else {
                            message.push(item);
                        }
                    }
                    chatService.addCompleteRequest(widget.viewModel.sessionId, request.message, request.variableData, request.attempt, {
                        message,
                        result: request.response?.result,
                        followups: request.response?.followups
                    });
                    if (workingSetInputs.size) {
                        for (const reference of workingSetInputs) {
                            chatEditingService.currentEditingSessionObs.get()?.addFileToWorkingSet(reference);
                        }
                    }
                    else {
                        for (const { reference } of request.response?.contentReferences ?? []) {
                            if (URI.isUri(reference) && [Schemas.file, Schemas.vscodeRemote].includes(reference.scheme)) {
                                chatEditingService.currentEditingSessionObs.get()?.addFileToWorkingSet(reference);
                            }
                        }
                    }
                }
                widget.focusInput();
            }
        }
    });
}
function splitMarkdownAndCodeBlocks(markdown) {
    const lexer = new marked.Lexer();
    const tokens = lexer.lex(markdown);
    const splitContent = [];
    let markdownPart = '';
    tokens.forEach((token) => {
        if (token.type === 'code') {
            if (markdownPart.trim()) {
                splitContent.push({ type: 'markdown', content: markdownPart });
                markdownPart = '';
            }
            splitContent.push({
                type: 'code',
                language: token.lang || '',
                content: token.text,
            });
        }
        else {
            markdownPart += token.raw;
        }
    });
    if (markdownPart.trim()) {
        splitContent.push({ type: 'markdown', content: markdownPart });
    }
    return splitContent;
}

export { MarkUnhelpfulActionId, registerChatTitleActions };
