import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { addDisposableListener, $, EventType, getWindow, EventHelper } from 'vscode/vscode/vs/base/browser/dom';
import { StandardMouseEvent } from 'vscode/vscode/vs/base/browser/mouseEvent';
import { getDefaultHoverDelegate } from 'vscode/vscode/vs/base/browser/ui/hover/hoverDelegateFactory';
import { Lazy } from 'vscode/vscode/vs/base/common/lazy';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { generateUuid } from 'vscode/vscode/vs/base/common/uuid';
import { ICodeEditorService } from 'vscode/vscode/vs/editor/browser/services/codeEditorService';
import { EditorContextKeys } from 'vscode/vscode/vs/editor/common/editorContextKeys';
import { SymbolKinds } from 'vscode/vscode/vs/editor/common/languages';
import { ILanguageService } from 'vscode/vscode/vs/editor/common/languages/language';
import { getIconClasses } from 'vscode/vscode/vs/editor/common/services/getIconClasses';
import { ILanguageFeaturesService } from 'vscode/vscode/vs/editor/common/services/languageFeatures';
import { IModelService } from 'vscode/vscode/vs/editor/common/services/model';
import { ITextModelService } from 'vscode/vscode/vs/editor/common/services/resolverService';
import { DefinitionAction } from 'vscode/vscode/vs/editor/contrib/gotoSymbol/browser/goToCommands';
import { localize, localize2 } from 'vscode/vscode/vs/nls';
import { createAndFillInContextMenuActions } from 'vscode/vscode/vs/platform/actions/browser/menuEntryActionViewItem';
import { MenuId, registerAction2, Action2 } from 'vscode/vscode/vs/platform/actions/common/actions';
import { IMenuService } from 'vscode/vscode/vs/platform/actions/common/actions.service';
import { IClipboardService } from 'vscode/vscode/vs/platform/clipboard/common/clipboardService.service';
import { ICommandService } from 'vscode/vscode/vs/platform/commands/common/commands.service';
import { RawContextKey } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IContextKeyService } from 'vscode/vscode/vs/platform/contextkey/common/contextkey.service';
import { IContextMenuService } from 'vscode/vscode/vs/platform/contextview/browser/contextView.service';
import { FileKind } from 'vscode/vscode/vs/platform/files/common/files';
import { IFileService } from 'vscode/vscode/vs/platform/files/common/files.service';
import { IHoverService } from 'vscode/vscode/vs/platform/hover/browser/hover.service';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { ILabelService } from 'vscode/vscode/vs/platform/label/common/label.service';
import { ITelemetryService } from 'vscode/vscode/vs/platform/telemetry/common/telemetry.service';
import { fillEditorsDragData } from 'vscode/vscode/vs/workbench/browser/dnd';
import { ResourceContextKey } from 'vscode/vscode/vs/workbench/common/contextkeys';
import { SIDE_GROUP } from 'vscode/vscode/vs/workbench/services/editor/common/editorService';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService.service';
import { ExplorerFolderContext } from 'vscode/vscode/vs/workbench/contrib/files/common/files';
import { IChatVariablesService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatVariables.service';
import { IChatWidgetService } from 'vscode/vscode/vs/workbench/contrib/chat/browser/chat.service';
import { IChatMarkdownAnchorService } from './chatContentParts/chatMarkdownAnchorService.service.js';

var InlineAnchorWidget_1;
const chatResourceContextKey = ( (new RawContextKey(
    'chatAnchorResource',
    undefined,
    { type: 'URI', description: ( localize(
        10861,
        "The full value of the chat anchor resource, including scheme and path"
    )) }
)));
let InlineAnchorWidget = class InlineAnchorWidget extends Disposable {
    static { InlineAnchorWidget_1 = this; }
    static { this.className = 'chat-inline-anchor-widget'; }
    constructor(element, inlineReference, originalContextKeyService, contextMenuService, fileService, hoverService, instantiationService, labelService, languageFeaturesService, languageService, menuService, modelService, textModelService, telemetryService) {
        super();
        this.element = element;
        this.inlineReference = inlineReference;
        this._isDisposed = false;
        this.data = 'uri' in inlineReference.inlineReference
            ? inlineReference.inlineReference
            : 'name' in inlineReference.inlineReference
                ? { kind: 'symbol', symbol: inlineReference.inlineReference }
                : { uri: inlineReference.inlineReference };
        const contextKeyService = this._register(originalContextKeyService.createScoped(element));
        this._chatResourceContext = chatResourceContextKey.bindTo(contextKeyService);
        const anchorId = ( (new Lazy(generateUuid)));
        element.classList.add(InlineAnchorWidget_1.className, 'show-file-icons');
        let iconText;
        let iconClasses;
        let location;
        let contextMenuId;
        let contextMenuArg;
        let updateContextKeys;
        if (this.data.kind === 'symbol') {
            location = this.data.symbol.location;
            contextMenuId = MenuId.ChatInlineSymbolAnchorContext;
            contextMenuArg = location;
            iconText = this.data.symbol.name;
            iconClasses = ['codicon', ...getIconClasses(modelService, languageService, undefined, undefined, SymbolKinds.toIcon(this.data.symbol.kind))];
            const providerContexts = [
                [EditorContextKeys.hasDefinitionProvider.bindTo(contextKeyService), languageFeaturesService.definitionProvider],
                [EditorContextKeys.hasReferenceProvider.bindTo(contextKeyService), languageFeaturesService.referenceProvider],
                [EditorContextKeys.hasImplementationProvider.bindTo(contextKeyService), languageFeaturesService.implementationProvider],
                [EditorContextKeys.hasTypeDefinitionProvider.bindTo(contextKeyService), languageFeaturesService.typeDefinitionProvider],
            ];
            updateContextKeys = async () => {
                const modelRef = await textModelService.createModelReference(location.uri);
                try {
                    if (this._isDisposed) {
                        return;
                    }
                    const model = modelRef.object.textEditorModel;
                    for (const [contextKey, registry] of providerContexts) {
                        contextKey.set(( (registry.has(model))));
                    }
                }
                finally {
                    modelRef.dispose();
                }
            };
            this._register(addDisposableListener(element, 'click', () => {
                telemetryService.publicLog2('chat.inlineAnchor.openSymbol', {
                    anchorId: anchorId.value
                });
            }));
        }
        else {
            location = this.data;
            contextMenuId = MenuId.ChatInlineResourceAnchorContext;
            contextMenuArg = location.uri;
            const label = labelService.getUriBasenameLabel(location.uri);
            iconText = location.range && this.data.kind !== 'symbol' ?
                `${label}#${location.range.startLineNumber}-${location.range.endLineNumber}` :
                label;
            const fileKind = location.uri.path.endsWith('/') ? FileKind.FOLDER : FileKind.FILE;
            iconClasses = getIconClasses(modelService, languageService, location.uri, fileKind);
            const isFolderContext = ExplorerFolderContext.bindTo(contextKeyService);
            fileService.stat(location.uri)
                .then(stat => {
                isFolderContext.set(stat.isDirectory);
            })
                .catch(() => { });
            this._register(addDisposableListener(element, 'click', () => {
                telemetryService.publicLog2('chat.inlineAnchor.openResource', {
                    anchorId: anchorId.value
                });
            }));
        }
        const resourceContextKey = this._register(( (new ResourceContextKey(contextKeyService, fileService, languageService, modelService))));
        resourceContextKey.set(location.uri);
        this._chatResourceContext.set(( (location.uri.toString())));
        const iconEl = $('span.icon');
        iconEl.classList.add(...iconClasses);
        element.replaceChildren(iconEl, $('span.icon-label', {}, iconText));
        const fragment = location.range ? `${location.range.startLineNumber},${location.range.startColumn}` : '';
        element.setAttribute('data-href', ( ((fragment ? location.uri.with({ fragment }) : location.uri).toString())));
        this._register(addDisposableListener(element, EventType.CONTEXT_MENU, async (domEvent) => {
            const event = ( (new StandardMouseEvent(getWindow(domEvent), domEvent)));
            EventHelper.stop(domEvent, true);
            try {
                await updateContextKeys?.();
            }
            catch (e) {
                console.error(e);
            }
            if (this._isDisposed) {
                return;
            }
            contextMenuService.showContextMenu({
                contextKeyService,
                getAnchor: () => event,
                getActions: () => {
                    const menu = menuService.getMenuActions(contextMenuId, contextKeyService, { arg: contextMenuArg });
                    const primary = [];
                    createAndFillInContextMenuActions(menu, primary);
                    return primary;
                },
            });
        }));
        const relativeLabel = labelService.getUriLabel(location.uri, { relative: true });
        this._register(hoverService.setupManagedHover(getDefaultHoverDelegate('element'), element, relativeLabel));
        element.draggable = true;
        this._register(addDisposableListener(element, 'dragstart', e => {
            instantiationService.invokeFunction(accessor => fillEditorsDragData(accessor, [location.uri], e));
            e.dataTransfer?.setDragImage(element, 0, 0);
        }));
    }
    dispose() {
        this._isDisposed = true;
        super.dispose();
    }
    getHTMLElement() {
        return this.element;
    }
};
InlineAnchorWidget = InlineAnchorWidget_1 = ( (__decorate([
    ( (__param(2, IContextKeyService))),
    ( (__param(3, IContextMenuService))),
    ( (__param(4, IFileService))),
    ( (__param(5, IHoverService))),
    ( (__param(6, IInstantiationService))),
    ( (__param(7, ILabelService))),
    ( (__param(8, ILanguageFeaturesService))),
    ( (__param(9, ILanguageService))),
    ( (__param(10, IMenuService))),
    ( (__param(11, IModelService))),
    ( (__param(12, ITextModelService))),
    ( (__param(13, ITelemetryService)))
], InlineAnchorWidget)));
registerAction2(class AddFileToChatAction extends Action2 {
    static { this.id = 'chat.inlineResourceAnchor.addFileToChat'; }
    constructor() {
        super({
            id: AddFileToChatAction.id,
            title: ( localize2(10862, "Add File to Chat")),
            menu: [{
                    id: MenuId.ChatInlineResourceAnchorContext,
                    group: 'chat',
                    order: 1,
                    when: ( (ExplorerFolderContext.negate())),
                }]
        });
    }
    async run(accessor, resource) {
        const chatWidgetService = accessor.get(IChatWidgetService);
        const variablesService = accessor.get(IChatVariablesService);
        const widget = chatWidgetService.lastFocusedWidget;
        if (!widget) {
            return;
        }
        variablesService.attachContext('file', resource, widget.location);
    }
});
registerAction2(class CopyResourceAction extends Action2 {
    static { this.id = 'chat.inlineResourceAnchor.copyResource'; }
    constructor() {
        super({
            id: CopyResourceAction.id,
            title: ( localize2(10863, "Copy")),
            f1: false,
            precondition: chatResourceContextKey,
            keybinding: {
                weight: 200 ,
                primary: 2048  | 33 ,
            }
        });
    }
    async run(accessor) {
        const chatWidgetService = accessor.get(IChatMarkdownAnchorService);
        const clipboardService = accessor.get(IClipboardService);
        const anchor = chatWidgetService.lastFocusedAnchor;
        if (!anchor) {
            return;
        }
        const resource = anchor.data.kind === 'symbol' ? anchor.data.symbol.location.uri : anchor.data.uri;
        clipboardService.writeResources([resource]);
    }
});
registerAction2(class OpenToSideResourceAction extends Action2 {
    static { this.id = 'chat.inlineResourceAnchor.openToSide'; }
    constructor() {
        super({
            id: OpenToSideResourceAction.id,
            title: ( localize2(10864, "Open to the Side")),
            f1: false,
            precondition: chatResourceContextKey,
            keybinding: {
                weight: 400  + 2,
                primary: 2048  | 3 ,
                mac: {
                    primary: 256  | 3
                },
            },
            menu: [{
                    id: MenuId.ChatInlineSymbolAnchorContext,
                    group: 'navigation',
                    order: 1
                }]
        });
    }
    async run(accessor) {
        const chatWidgetService = accessor.get(IChatMarkdownAnchorService);
        const editorService = accessor.get(IEditorService);
        const anchor = chatWidgetService.lastFocusedAnchor;
        if (!anchor) {
            return;
        }
        const input = anchor.data.kind === 'symbol'
            ? {
                resource: anchor.data.symbol.location.uri, options: {
                    selection: {
                        startColumn: anchor.data.symbol.location.range.startColumn,
                        startLineNumber: anchor.data.symbol.location.range.startLineNumber,
                    }
                }
            }
            : { resource: anchor.data.uri };
        await editorService.openEditors([input], SIDE_GROUP);
    }
});
registerAction2(class GoToDefinitionAction extends Action2 {
    static { this.id = 'chat.inlineSymbolAnchor.goToDefinition'; }
    constructor() {
        super({
            id: GoToDefinitionAction.id,
            title: {
                ...( localize2(10865, "Go to Definition")),
                mnemonicTitle: ( localize(10866, "Go to &&Definition")),
            },
            menu: [{
                    id: MenuId.ChatInlineSymbolAnchorContext,
                    group: '4_symbol_nav',
                    order: 1.1,
                    when: EditorContextKeys.hasDefinitionProvider,
                }]
        });
    }
    async run(accessor, location) {
        const editorService = accessor.get(ICodeEditorService);
        await openEditorWithSelection(editorService, location);
        const action = ( (new DefinitionAction(
            { openToSide: false, openInPeek: false, muteMessage: true },
            { title: { value: '', original: '' }, id: '', precondition: undefined }
        )));
        return action.run(accessor);
    }
});
async function openEditorWithSelection(editorService, location) {
    await editorService.openCodeEditor({
        resource: location.uri, options: {
            selection: {
                startColumn: location.range.startColumn,
                startLineNumber: location.range.startLineNumber,
            }
        }
    }, null);
}
async function runGoToCommand(accessor, command, location) {
    const editorService = accessor.get(ICodeEditorService);
    const commandService = accessor.get(ICommandService);
    await openEditorWithSelection(editorService, location);
    return commandService.executeCommand(command);
}
registerAction2(class GoToTypeDefinitionsAction extends Action2 {
    static { this.id = 'chat.inlineSymbolAnchor.goToTypeDefinitions'; }
    constructor() {
        super({
            id: GoToTypeDefinitionsAction.id,
            title: {
                ...( localize2(10867, "Go to Type Definitions")),
                mnemonicTitle: ( localize(10868, "Go to &&Type Definitions")),
            },
            menu: [{
                    id: MenuId.ChatInlineSymbolAnchorContext,
                    group: '4_symbol_nav',
                    order: 1.1,
                    when: EditorContextKeys.hasTypeDefinitionProvider,
                },]
        });
    }
    async run(accessor, location) {
        return runGoToCommand(accessor, 'editor.action.goToTypeDefinition', location);
    }
});
registerAction2(class GoToImplementations extends Action2 {
    static { this.id = 'chat.inlineSymbolAnchor.goToImplementations'; }
    constructor() {
        super({
            id: GoToImplementations.id,
            title: {
                ...( localize2(10869, "Go to Implementations")),
                mnemonicTitle: ( localize(10870, "Go to &&Implementations")),
            },
            menu: [{
                    id: MenuId.ChatInlineSymbolAnchorContext,
                    group: '4_symbol_nav',
                    order: 1.2,
                    when: EditorContextKeys.hasImplementationProvider,
                },]
        });
    }
    async run(accessor, location) {
        return runGoToCommand(accessor, 'editor.action.goToImplementation', location);
    }
});
registerAction2(class GoToReferencesAction extends Action2 {
    static { this.id = 'chat.inlineSymbolAnchor.goToReferences'; }
    constructor() {
        super({
            id: GoToReferencesAction.id,
            title: {
                ...( localize2(10871, "Go to References")),
                mnemonicTitle: ( localize(10872, "Go to &&References")),
            },
            menu: [{
                    id: MenuId.ChatInlineSymbolAnchorContext,
                    group: '4_symbol_nav',
                    order: 1.3,
                    when: EditorContextKeys.hasReferenceProvider,
                },]
        });
    }
    async run(accessor, location) {
        return runGoToCommand(accessor, 'editor.action.goToReferences', location);
    }
});

export { InlineAnchorWidget };
